@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-encodings-dialog.c 5786 2007-08-12 12:36:30Z vgeddes $
+ * $Id: gedit-encodings-dialog.c 6727 2008-12-25 21:02:48Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,7 +34,7 @@
 
 #include <string.h>
 
-#include <glib/gslist.h>
+#include <glib.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
@@ -44,6 +44,7 @@
 #include "gedit-utils.h"
 #include "gedit-debug.h"
 #include "gedit-help.h"
+#include "gedit-dirs.h"
 
 #define GEDIT_ENCODINGS_DIALOG_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), \
 						   GEDIT_TYPE_ENCODINGS_DIALOG,           \
@@ -300,6 +301,11 @@ gedit_encodings_dialog_init (GeditEncodingsDialog *dlg)
 	GtkWidget *error_widget;
 	int i;
 	gboolean ret;
+	gchar *file;
+	gchar *root_objects[] = {
+		"encodings-dialog-contents",
+		NULL
+	};
 
 	dlg->priv = GEDIT_ENCODINGS_DIALOG_GET_PRIVATE (dlg);
 	
@@ -330,15 +336,17 @@ gedit_encodings_dialog_init (GeditEncodingsDialog *dlg)
 			  G_CALLBACK (response_handler),
 			  dlg);
 
-	ret = gedit_utils_get_glade_widgets (GEDIT_GLADEDIR "gedit-encodings-dialog.glade",
-					     "encodings-dialog-contents",
-					     &error_widget,
-					     "encodings-dialog-contents", &content,
-					     "add-button", &dlg->priv->add_button,
-					     "remove-button", &dlg->priv->remove_button,
-					     "available-treeview", &dlg->priv->available_treeview,
-					     "displayed-treeview", &dlg->priv->displayed_treeview,
-					     NULL);
+	file = gedit_dirs_get_ui_file ("gedit-encodings-dialog.ui");
+	ret = gedit_utils_get_ui_objects (file,
+					  root_objects,
+					  &error_widget,
+					  "encodings-dialog-contents", &content,
+					  "add-button", &dlg->priv->add_button,
+					  "remove-button", &dlg->priv->remove_button,
+					  "available-treeview", &dlg->priv->available_treeview,
+					  "displayed-treeview", &dlg->priv->displayed_treeview,
+					  NULL);
+	g_free (file);
 
 	if (!ret)
 	{
@@ -353,6 +361,7 @@ gedit_encodings_dialog_init (GeditEncodingsDialog *dlg)
 
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox),
 			    content, TRUE, TRUE, 0);
+	g_object_unref (content);
 	gtk_container_set_border_width (GTK_CONTAINER (content), 5);			     
 
 	g_signal_connect (dlg->priv->add_button,
