@@ -27,7 +27,7 @@
  * list of people on the gedit Team.
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-commands-search.c,v 1.9 2006/04/21 08:14:55 paolo Exp $
+ * $Id: gedit-commands-search.c 5931 2007-09-25 20:05:40Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -84,14 +84,14 @@ last_search_data_set (LastSearchData    *data,
 	if (str != NULL && *str != '\0')
 	{
 		g_free (data->find_text);
-		data->find_text = g_strdup (str);
+		data->find_text = gedit_utils_unescape_search_text (str);
 	}
 
 	str = gedit_search_dialog_get_replace_text (dialog);
 	if (str != NULL && *str != '\0')
 	{
 		g_free (data->replace_text);
-		data->replace_text = g_strdup (str);
+		data->replace_text = gedit_utils_unescape_search_text (str);
 	}
 
 	data->match_case = gedit_search_dialog_get_match_case (dialog);
@@ -513,7 +513,7 @@ search_dialog_destroyed (GeditWindow       *window,
 }
 
 void
-gedit_cmd_search_find (GtkAction   *action,
+_gedit_cmd_search_find (GtkAction   *action,
 		       GeditWindow *window)
 {
 	gpointer data;
@@ -536,7 +536,8 @@ gedit_cmd_search_find (GtkAction   *action,
 		 * it is already open
 		 */
 
-		gtk_window_present (GTK_WINDOW (data));
+		gedit_search_dialog_present_with_time (GEDIT_SEARCH_DIALOG (data),
+						       GDK_CURRENT_TIME);
 
 		return;
 	}
@@ -594,7 +595,7 @@ gedit_cmd_search_find (GtkAction   *action,
 }
 
 void
-gedit_cmd_search_replace (GtkAction   *action,
+_gedit_cmd_search_replace (GtkAction   *action,
 			  GeditWindow *window)
 {
 	gpointer data;
@@ -618,7 +619,8 @@ gedit_cmd_search_replace (GtkAction   *action,
 			gedit_search_dialog_set_show_replace (GEDIT_SEARCH_DIALOG (data),
 							      TRUE);
 
-		gtk_window_present (GTK_WINDOW (data));
+		gedit_search_dialog_present_with_time (GEDIT_SEARCH_DIALOG (data),
+						       GDK_CURRENT_TIME);
 
 		return;
 	}
@@ -681,15 +683,18 @@ do_find_again (GeditWindow *window,
 	       gboolean     backward)
 {
 	GeditView *active_view;
+	GtkTextBuffer *buffer;
 	gboolean wrap_around = TRUE;
 	LastSearchData *data;
 	
 	active_view = gedit_window_get_active_view (window);
 	g_return_if_fail (active_view != NULL);
 
-	data = g_object_get_data (G_OBJECT (gtk_text_view_get_buffer (active_view)), 
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (active_view));
+
+	data = g_object_get_data (G_OBJECT (buffer), 
 				  GEDIT_LAST_SEARCH_DATA_KEY);
-					    
+
 	if (data != NULL)
 		wrap_around = data->wrap_around;					    
 
@@ -699,7 +704,7 @@ do_find_again (GeditWindow *window,
 }				 
 
 void
-gedit_cmd_search_find_next (GtkAction   *action,
+_gedit_cmd_search_find_next (GtkAction   *action,
 			    GeditWindow *window)
 {
 	gedit_debug (DEBUG_COMMANDS);
@@ -708,7 +713,7 @@ gedit_cmd_search_find_next (GtkAction   *action,
 }
 
 void
-gedit_cmd_search_find_prev (GtkAction   *action,
+_gedit_cmd_search_find_prev (GtkAction   *action,
 			    GeditWindow *window)
 {
 	gedit_debug (DEBUG_COMMANDS);
@@ -717,7 +722,7 @@ gedit_cmd_search_find_prev (GtkAction   *action,
 }
 
 void
-gedit_cmd_search_clear_highlight (GtkAction   *action,
+_gedit_cmd_search_clear_highlight (GtkAction   *action,
 				  GeditWindow *window)
 {
 	GeditDocument *doc;
@@ -731,7 +736,7 @@ gedit_cmd_search_clear_highlight (GtkAction   *action,
 }
 
 void
-gedit_cmd_search_goto_line (GtkAction   *action,
+_gedit_cmd_search_goto_line (GtkAction   *action,
 			    GeditWindow *window)
 {
 	GeditView *active_view;
@@ -742,6 +747,11 @@ gedit_cmd_search_goto_line (GtkAction   *action,
 	if (active_view == NULL)
 		return;
 
+	/* Focus the view if needed: we need to focus the view otherwise 
+	   activating the binding for goto line has no effect */
+	gtk_widget_grab_focus (GTK_WIDGET (active_view));
+
+
 	/* goto line is builtin in GeditView, just activate
 	 * the corrisponding binding.
 	 */
@@ -749,3 +759,27 @@ gedit_cmd_search_goto_line (GtkAction   *action,
 			       GDK_i,
 			       GDK_CONTROL_MASK);
 }
+
+void
+_gedit_cmd_search_incremental_search (GtkAction   *action,
+				      GeditWindow *window)
+{
+	GeditView *active_view;
+
+	gedit_debug (DEBUG_COMMANDS);
+
+	active_view = gedit_window_get_active_view (window);
+	if (active_view == NULL)
+		return;
+
+	/* Focus the view if needed: we need to focus the view otherwise 
+	   activating the binding for incremental search has no effect */
+	gtk_widget_grab_focus (GTK_WIDGET (active_view));
+	
+	/* incremental search is builtin in GeditView, just activate
+	 * the corrisponding binding.
+	 */
+	gtk_bindings_activate (GTK_OBJECT (active_view),
+			       GDK_k,
+			       GDK_CONTROL_MASK);
+}
