@@ -41,8 +41,11 @@
 #include "gedit-file-browser-marshal.h"
 #include "gedit-file-browser-enum-types.h"
 
-#define GEDIT_FILE_BROWSER_WIDGET_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GEDIT_TYPE_FILE_BROWSER_WIDGET, GeditFileBrowserWidgetPrivate))
-#define XML_UI_FILE GEDIT_UIDIR "/gedit-file-browser-widget-ui.xml"
+#define GEDIT_FILE_BROWSER_WIDGET_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), \
+						      GEDIT_TYPE_FILE_BROWSER_WIDGET, \
+						      GeditFileBrowserWidgetPrivate))
+
+#define XML_UI_FILE "gedit-file-browser-widget-ui.xml"
 #define LOCATION_DATA_KEY "gedit-file-browser-widget-location"
 
 enum 
@@ -96,6 +99,7 @@ typedef struct
 	gulong id;
 	GeditFileBrowserWidgetFilterFunc func;
 	gpointer user_data;
+	GDestroyNotify destroy_notify;
 } FilterFunc;
 
 typedef struct 
@@ -149,6 +153,8 @@ struct _GeditFileBrowserWidgetPrivate
 	gboolean enable_delete;
 	
 	GCancellable *cancellable;
+	
+	GdkCursor *busy_cursor;
 };
 
 static void set_enable_delete		       (GeditFileBrowserWidget *obj,
@@ -246,7 +252,8 @@ free_name_icon (gpointer data)
 static FilterFunc *
 filter_func_new (GeditFileBrowserWidget * obj,
 		 GeditFileBrowserWidgetFilterFunc func,
-		 gpointer user_data)
+		 gpointer user_data,
+		 GDestroyNotify notify)
 {
 	FilterFunc *result;
 
@@ -255,7 +262,7 @@ filter_func_new (GeditFileBrowserWidget * obj,
 	result->id = ++obj->priv->filter_id;
 	result->func = func;
 	result->user_data = user_data;
-
+	result->destroy_notify = notify;
 	return result;
 }
 
@@ -337,13 +344,16 @@ gedit_file_browser_widget_finalize (GObject * object)
 		location_free ((Location *) (loc->data));
 
 	if (obj->priv->current_location_menu_item)
-		gtk_widget_unref (obj->priv->current_location_menu_item);
+		g_object_unref (obj->priv->current_location_menu_item);
 
 	g_list_free (obj->priv->locations);
 
 	g_hash_table_destroy (obj->priv->bookmarks_hash);
 	
 	cancel_async_operation (obj);
+	
+	gdk_cursor_unref (obj->priv->busy_cursor);
+
 	G_OBJECT_CLASS (gedit_file_browser_widget_parent_class)->finalize (object);
 }
 
@@ -423,7 +433,7 @@ gedit_file_browser_widget_class_init (GeditFileBrowserWidgetClass * klass)
 			  G_STRUCT_OFFSET (GeditFileBrowserWidgetClass,
 					   uri_activated), NULL, NULL,
 			  g_cclosure_marshal_VOID__STRING, G_TYPE_NONE, 1,
-			  GTK_TYPE_STRING);
+			  G_TYPE_STRING);
 	signals[ERROR] =
 	    g_signal_new ("error", G_OBJECT_CLASS_TYPE (object_class),
 			  G_SIGNAL_RUN_LAST,
@@ -443,7 +453,7 @@ gedit_file_browser_widget_class_init (GeditFileBrowserWidgetClass * klass)
 	                  G_TYPE_BOOLEAN,
 	                  2,
 	                  G_TYPE_OBJECT,
-	                  GTK_TYPE_POINTER);
+	                  G_TYPE_POINTER);
 
 	signals[CONFIRM_NO_TRASH] =
 	    g_signal_new ("confirm-no-trash", G_OBJECT_CLASS_TYPE (object_class),
@@ -455,7 +465,7 @@ gedit_file_browser_widget_class_init (GeditFileBrowserWidgetClass * klass)
 	                  gedit_file_browser_marshal_BOOL__POINTER,
 	                  G_TYPE_BOOLEAN,
 	                  1,
-	                  GTK_TYPE_POINTER);
+	                  G_TYPE_POINTER);
 
 	g_type_class_add_private (object_class,
 				  sizeof (GeditFileBrowserWidgetPrivate));
@@ -751,6 +761,9 @@ create_combo (GeditFileBrowserWidget * obj)
 	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (obj->priv->combo),
 				       renderer, "text", COLUMN_NAME);
 
+	g_object_set (renderer, "ellipsize-set", TRUE,
+		      "ellipsize", PANGO_ELLIPSIZE_END, NULL);
+
 	gtk_box_pack_start (GTK_BOX (obj), GTK_WIDGET (obj->priv->combo),
 			    FALSE, FALSE, 0);
 
@@ -825,7 +838,8 @@ static const GtkToggleActionEntry tree_actions_toggle[] =
 };
 
 static void
-create_toolbar (GeditFileBrowserWidget * obj)
+create_toolbar (GeditFileBrowserWidget * obj,
+		const gchar *data_dir)
 {
 	GtkUIManager *manager;
 	GError *error = NULL;
@@ -833,11 +847,15 @@ create_toolbar (GeditFileBrowserWidget * obj)
 	GtkWidget *toolbar;
 	GtkWidget *widget;
 	GtkAction *action;
+	gchar *ui_file;
 
 	manager = gtk_ui_manager_new ();
 	obj->priv->manager = manager;
 
-	gtk_ui_manager_add_ui_from_file (manager, XML_UI_FILE, &error);
+	ui_file = g_build_filename (data_dir, XML_UI_FILE, NULL);
+	gtk_ui_manager_add_ui_from_file (manager, ui_file, &error);
+
+	g_free (ui_file);
 
 	if (error != NULL) {
 		g_warning ("Error in adding ui from file %s: %s",
@@ -919,67 +937,48 @@ create_toolbar (GeditFileBrowserWidget * obj)
 	obj->priv->location_previous_menu = gtk_menu_new ();
 	gtk_widget_show (obj->priv->location_previous_menu);
 
-	widget =
-	    GTK_WIDGET (gtk_menu_tool_button_new_from_stock
-			(GTK_STOCK_GO_BACK));
+	widget = GTK_WIDGET (gtk_menu_tool_button_new_from_stock (GTK_STOCK_GO_BACK));
 	gtk_menu_tool_button_set_menu (GTK_MENU_TOOL_BUTTON (widget),
 				       obj->priv->location_previous_menu);
 
 	g_object_set (widget, "label", _("Previous location"), NULL);
+	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (widget),
+					_("Go to previous location"));
+	gtk_menu_tool_button_set_arrow_tooltip_text (GTK_MENU_TOOL_BUTTON (widget),
+						     _("Go to a previously opened location"));
 
-	gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (widget),
-				   GTK_TOOLBAR (toolbar)->tooltips,
-				   _("Go to previous location"), NULL);
-	gtk_menu_tool_button_set_arrow_tooltip (GTK_MENU_TOOL_BUTTON
-						(widget),
-						GTK_TOOLBAR (toolbar)->
-						tooltips,
-						_("Go to a previously opened location"),
-						NULL);
-
-	action =
-	    gtk_action_group_get_action (obj->priv->action_group_sensitive,
-					 "DirectoryPrevious");
+	action = gtk_action_group_get_action (obj->priv->action_group_sensitive,
+					      "DirectoryPrevious");
 	g_object_set (action, "is_important", TRUE, "short_label",
 		      _("Previous location"), NULL);
 	gtk_action_connect_proxy (action, widget);
-	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (widget),
-			    0);
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (widget), 0);
 
 	/* Next directory menu tool item */
 	obj->priv->location_next_menu = gtk_menu_new ();
 	gtk_widget_show (obj->priv->location_next_menu);
 
-	widget =
-	    GTK_WIDGET (gtk_menu_tool_button_new_from_stock
-			(GTK_STOCK_GO_FORWARD));
+	widget = GTK_WIDGET (gtk_menu_tool_button_new_from_stock (GTK_STOCK_GO_FORWARD));
 	gtk_menu_tool_button_set_menu (GTK_MENU_TOOL_BUTTON (widget),
 				       obj->priv->location_next_menu);
 
 	g_object_set (widget, "label", _("Next location"), NULL);
+	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (widget),
+					_("Go to next location"));
+	gtk_menu_tool_button_set_arrow_tooltip_text (GTK_MENU_TOOL_BUTTON (widget),
+						     _("Go to a previously opened location"));
 
-	gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (widget),
-				   GTK_TOOLBAR (toolbar)->tooltips,
-				   _("Go to next location"), NULL);
-	gtk_menu_tool_button_set_arrow_tooltip (GTK_MENU_TOOL_BUTTON
-						(widget),
-						GTK_TOOLBAR (toolbar)->
-						tooltips,
-						_
-						("Go to a previously opened location"),
-						NULL);
-
-	action =
-	    gtk_action_group_get_action (obj->priv->action_group_sensitive,
-					 "DirectoryNext");
+	action = gtk_action_group_get_action (obj->priv->action_group_sensitive,
+					      "DirectoryNext");
 	g_object_set (action, "is_important", TRUE, "short_label",
 		      _("Previous location"), NULL);
 	gtk_action_connect_proxy (action, widget);
-	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (widget),
-			    1);
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (widget), 1);
 
 	gtk_box_pack_start (GTK_BOX (obj), toolbar, FALSE, FALSE, 0);
 	gtk_widget_show (toolbar);
+	
+	set_enable_delete (obj, obj->priv->enable_delete);
 }
 
 static void 
@@ -989,6 +988,9 @@ set_enable_delete (GeditFileBrowserWidget *obj,
 	GtkAction *action;
 	obj->priv->enable_delete = enable;
 	
+	if (obj->priv->action_group_selection == NULL)
+		return;
+	
 	action =
 	    gtk_action_group_get_action (obj->priv->action_group_selection,
 					 "FileDelete");
@@ -1077,6 +1079,28 @@ init_bookmarks_hash (GeditFileBrowserWidget * obj)
 		          obj);
 }
 
+static void
+on_begin_loading (GeditFileBrowserStore  *model, 
+		  GtkTreeIter            *iter,
+		  GeditFileBrowserWidget *obj)
+{
+	if (!GDK_IS_WINDOW (GTK_WIDGET (obj->priv->treeview)->window))
+		return;
+
+	gdk_window_set_cursor (GTK_WIDGET (obj)->window, obj->priv->busy_cursor);
+}
+
+static void
+on_end_loading (GeditFileBrowserStore  *model, 
+		GtkTreeIter            *iter,
+		GeditFileBrowserWidget *obj)
+{
+	if (!GDK_IS_WINDOW (GTK_WIDGET (obj->priv->treeview)->window))
+		return;
+
+	gdk_window_set_cursor (GTK_WIDGET (obj)->window, NULL);
+}
+
 static void
 create_tree (GeditFileBrowserWidget * obj)
 {
@@ -1126,6 +1150,18 @@ create_tree (GeditFileBrowserWidget * obj)
 	g_signal_connect (obj->priv->file_store, "notify::filter-mode",
 			  G_CALLBACK (on_filter_mode_changed), obj);
 
+	g_signal_connect (obj->priv->file_store, "notify::virtual-root",
+			  G_CALLBACK (on_virtual_root_changed), obj);
+			  
+	g_signal_connect (obj->priv->file_store, "begin-loading",
+			  G_CALLBACK (on_begin_loading), obj);
+			  
+	g_signal_connect (obj->priv->file_store, "end-loading",
+			  G_CALLBACK (on_end_loading), obj);
+
+	g_signal_connect (obj->priv->file_store, "error",
+			  G_CALLBACK (on_file_store_error), obj);
+	      
 	init_bookmarks_hash (obj);
 
 	gtk_widget_show (sw);
@@ -1174,14 +1210,9 @@ gedit_file_browser_widget_init (GeditFileBrowserWidget * obj)
 			                                   g_object_unref,
 			                                   free_name_icon);
 
-	create_toolbar (obj);
-	create_combo (obj);
-	create_tree (obj);
-	create_filter (obj);
-
 	gtk_box_set_spacing (GTK_BOX (obj), 3);
-
-	gedit_file_browser_widget_show_bookmarks (obj);
+	
+	obj->priv->busy_cursor = gdk_cursor_new (GDK_WATCH);
 }
 
 /* Private */
@@ -1513,13 +1544,13 @@ jump_to_location (GeditFileBrowserWidget * obj, GList * item,
 			gtk_menu_shell_prepend (GTK_MENU_SHELL (menu_to),
 						widget);
 
-			gtk_widget_unref (widget);
+			g_object_unref (widget);
 		}
 
 		widget = GTK_WIDGET (child->data);
 
 		/* Make sure the widget isn't destroyed when removed */
-		gtk_widget_ref (widget);
+		g_object_ref (widget);
 		gtk_container_remove (GTK_CONTAINER (menu_from), widget);
 
 		obj->priv->current_location_menu_item = widget;
@@ -1661,6 +1692,7 @@ set_filter_pattern_real (GeditFileBrowserWidget * obj,
 			obj->priv->glob_filter_id =
 			    gedit_file_browser_widget_add_filter (obj,
 								  filter_glob,
+								  NULL,
 								  NULL);
 	}
 
@@ -1688,11 +1720,18 @@ set_filter_pattern_real (GeditFileBrowserWidget * obj,
 /* Public */
 
 GtkWidget *
-gedit_file_browser_widget_new ()
+gedit_file_browser_widget_new (const gchar *data_dir)
 {
 	GeditFileBrowserWidget *obj =
 	    g_object_new (GEDIT_TYPE_FILE_BROWSER_WIDGET, NULL);
 
+	create_toolbar (obj, data_dir);
+	create_combo (obj);
+	create_tree (obj);
+	create_filter (obj);
+
+	gedit_file_browser_widget_show_bookmarks (obj);
+
 	return GTK_WIDGET (obj);
 }
 
@@ -1713,6 +1752,24 @@ gedit_file_browser_widget_show_bookmarks (GeditFileBrowserWidget * obj)
 							   bookmarks_store));
 }
 
+static void
+show_files_real (GeditFileBrowserWidget *obj,
+		 gboolean                do_root_changed)
+{
+	gedit_file_browser_view_set_model (obj->priv->treeview,
+					   GTK_TREE_MODEL (obj->priv->
+							   file_store));
+
+	if (do_root_changed)
+		on_virtual_root_changed (obj->priv->file_store, NULL, obj);
+}
+
+void
+gedit_file_browser_widget_show_files (GeditFileBrowserWidget * obj)
+{
+	show_files_real (obj, TRUE);
+}
+
 void
 gedit_file_browser_widget_set_root_and_virtual_root (GeditFileBrowserWidget *obj,
 						     gchar const *root,
@@ -1720,10 +1777,6 @@ gedit_file_browser_widget_set_root_and_virtual_root (GeditFileBrowserWidget *obj
 {
 	GeditFileBrowserStoreResult result;
 
-	gedit_file_browser_view_set_model (obj->priv->treeview,
-					   GTK_TREE_MODEL (obj->priv->
-							   file_store));
-
 	if (!virtual_root)
 		result =
 		    gedit_file_browser_store_set_root_and_virtual_root 
@@ -1734,7 +1787,7 @@ gedit_file_browser_widget_set_root_and_virtual_root (GeditFileBrowserWidget *obj
 		    (obj->priv->file_store, root, virtual_root);
 
 	if (result == GEDIT_FILE_BROWSER_STORE_RESULT_NO_CHANGE)
-		on_virtual_root_changed (obj->priv->file_store, NULL, obj);
+		show_files_real (obj, TRUE);
 }
 
 void
@@ -1801,14 +1854,15 @@ gedit_file_browser_widget_get_filter_entry (GeditFileBrowserWidget * obj)
 
 gulong
 gedit_file_browser_widget_add_filter (GeditFileBrowserWidget * obj,
-				      GeditFileBrowserWidgetFilterFunc
-				      func, gpointer user_data)
+				      GeditFileBrowserWidgetFilterFunc func, 
+				      gpointer user_data,
+				      GDestroyNotify notify)
 {
 	FilterFunc *f;
 	GtkTreeModel *model =
 	    gtk_tree_view_get_model (GTK_TREE_VIEW (obj->priv->treeview));
 
-	f = filter_func_new (obj, func, user_data);
+	f = filter_func_new (obj, func, user_data, notify);
 	obj->priv->filter_funcs =
 	    g_slist_append (obj->priv->filter_funcs, f);
 
@@ -1826,10 +1880,15 @@ gedit_file_browser_widget_remove_filter (GeditFileBrowserWidget * obj,
 	GSList *item;
 	FilterFunc *func;
 
-	for (item = obj->priv->filter_funcs; item; item = item->next) {
+	for (item = obj->priv->filter_funcs; item; item = item->next)
+	{
 		func = (FilterFunc *) (item->data);
 
-		if (func->id == id) {
+		if (func->id == id)
+		{
+			if (func->destroy_notify)
+				func->destroy_notify (func->user_data);
+
 			obj->priv->filter_funcs =
 			    g_slist_remove_link (obj->priv->filter_funcs,
 						 item);
@@ -2186,6 +2245,46 @@ activate_volume (GeditFileBrowserWidget *obj,
 	g_object_unref (volume);
 }
 
+void 
+gedit_file_browser_widget_refresh (GeditFileBrowserWidget *obj)
+{
+	GtkTreeModel *model =
+	    gtk_tree_view_get_model (GTK_TREE_VIEW (obj->priv->treeview));
+
+	if (GEDIT_IS_FILE_BROWSER_STORE (model))
+		gedit_file_browser_store_refresh (GEDIT_FILE_BROWSER_STORE
+						  (model));
+	else if (GEDIT_IS_FILE_BOOKMARKS_STORE (model)) {
+		g_hash_table_ref (obj->priv->bookmarks_hash);
+		g_hash_table_destroy (obj->priv->bookmarks_hash);
+
+		gedit_file_bookmarks_store_refresh
+		    (GEDIT_FILE_BOOKMARKS_STORE (model));
+	}
+}
+
+void 
+gedit_file_browser_widget_history_back (GeditFileBrowserWidget *obj)
+{
+	if (obj->priv->locations) {
+		if (obj->priv->current_location)
+			jump_to_location (obj,
+					  obj->priv->current_location->
+					  next, TRUE);
+		else {
+			jump_to_location (obj, obj->priv->locations, TRUE);
+		}
+	}
+}
+
+void
+gedit_file_browser_widget_history_forward (GeditFileBrowserWidget *obj)
+{
+	if (obj->priv->locations)
+		jump_to_location (obj, obj->priv->current_location->prev,
+				  FALSE);
+}
+
 /* Callbacks */
 static void
 on_bookmark_activated (GeditFileBrowserView   *tree_view,
@@ -2203,12 +2302,14 @@ on_bookmark_activated (GeditFileBrowserView   *tree_view,
 	if (flags & GEDIT_FILE_BOOKMARKS_STORE_IS_DRIVE)
 	{
 		/* handle a drive node */
+		gedit_file_browser_store_cancel_mount_operation (obj->priv->file_store);
 		activate_drive (obj, iter);
 		return;
 	}
 	else if (flags & GEDIT_FILE_BOOKMARKS_STORE_IS_VOLUME)
 	{
 		/* handle a volume node */
+		gedit_file_browser_store_cancel_mount_operation (obj->priv->file_store);
 		activate_volume (obj, iter);
 		return;
 	}
@@ -2289,6 +2390,12 @@ on_virtual_root_changed (GeditFileBrowserStore * model,
 	Location *loc;
 	GdkPixbuf *pixbuf;
 
+	if (gtk_tree_view_get_model (GTK_TREE_VIEW (obj->priv->treeview)) !=
+	    GTK_TREE_MODEL (obj->priv->file_store))
+	{
+		show_files_real (obj, FALSE);
+	}
+
 	if (gedit_file_browser_store_get_iter_virtual_root (model, &iter)) {
 		gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
 				    GEDIT_FILE_BROWSER_STORE_COLUMN_URI,
@@ -2428,16 +2535,6 @@ on_model_set (GObject * gobject, GParamSpec * arg1,
 			    g_signal_connect (gobject, "file-activated",
 					      G_CALLBACK
 					      (on_file_activated), obj));
-		add_signal (obj, model,
-			    g_signal_connect (model,
-					      "notify::virtual-root",
-					      G_CALLBACK
-					      (on_virtual_root_changed),
-					      obj));
-		add_signal (obj, model,
-			    g_signal_connect (model, "error",
-					      G_CALLBACK
-					      (on_file_store_error), obj));
 
 		add_signal (obj, model,
 			    g_signal_connect (model, "no-trash",
@@ -2720,24 +2817,14 @@ on_filter_mode_changed (GeditFileBrowserStore * model,
 static void
 on_action_directory_next (GtkAction * action, GeditFileBrowserWidget * obj)
 {
-	if (obj->priv->locations)
-		jump_to_location (obj, obj->priv->current_location->prev,
-				  FALSE);
+	gedit_file_browser_widget_history_forward (obj);
 }
 
 static void
 on_action_directory_previous (GtkAction * action,
 			      GeditFileBrowserWidget * obj)
 {
-	if (obj->priv->locations) {
-		if (obj->priv->current_location)
-			jump_to_location (obj,
-					  obj->priv->current_location->
-					  next, TRUE);
-		else {
-			jump_to_location (obj, obj->priv->locations, TRUE);
-		}
-	}
+	gedit_file_browser_widget_history_back (obj);
 }
 
 static void 
@@ -2818,19 +2905,7 @@ static void
 on_action_directory_refresh (GtkAction * action,
 			     GeditFileBrowserWidget * obj)
 {
-	GtkTreeModel *model =
-	    gtk_tree_view_get_model (GTK_TREE_VIEW (obj->priv->treeview));
-
-	if (GEDIT_IS_FILE_BROWSER_STORE (model))
-		gedit_file_browser_store_refresh (GEDIT_FILE_BROWSER_STORE
-						  (model));
-	else if (GEDIT_IS_FILE_BOOKMARKS_STORE (model)) {
-		g_hash_table_ref (obj->priv->bookmarks_hash);
-		g_hash_table_destroy (obj->priv->bookmarks_hash);
-
-		gedit_file_bookmarks_store_refresh
-		    (GEDIT_FILE_BOOKMARKS_STORE (model));
-	}
+	gedit_file_browser_widget_refresh (obj);
 }
 
 static gboolean
